'use strict';

const { validateValue } = require('@validatem/core');
const isString = require('@validatem/is-string');
const defaultTo = require('@validatem/default-to');
const either = require('@validatem/either');
const isFunction = require('@validatem/is-function');
const anyProperty = require('@validatem/any-property');
const matchesFormat = require('@validatem/matches-format');
const { HTTPException } = require('@local/http-errors');

module.exports = function useErrorHandler(app, userOptions) {
	const { debug, json, ...errorNumViews } = userOptions;
	const transformJson = validateValue(json, [
		defaultTo(error => error),
		isFunction,
	]);
	const views = validateValue(
		errorNumViews,
		anyProperty({
			key: [matchesFormat(/^[1-5][0-9][0-9]$/)],
			value: [either([isFunction, isString])],
		}),
	);
	// eslint-disable-next-line no-unused-vars
	app.use((error, req, res, next) => {
		if (!(error instanceof HTTPException)) {
			if (error.code != null) {
				// eslint-disable-next-line no-param-reassign
				error.originalCode = error.code;
			}
			// eslint-disable-next-line no-param-reassign
			error.code = 500;
		}
		const errorData = {
			status: error.status,
			code: error.code,
		};
		if (debug) {
			// eslint-disable-next-line no-console
			console.error({ ...errorData }, error);
			errorData.fullError = error;
		}
		if (req.accepts('html')) {
			res.status(error.code);
			if (debug) {
				res.send(`${error.code}\n${error}`);
			} else if (views[error.code]) {
				res.render(views[error.code], errorData);
			} else if (views[500]) {
				res.render(views[500], errorData);
			} else {
				res.send(error.status ?? 'Internal Server Error');
			}
		} else {
			res.status(error.code).json(transformJson(errorData));
		}
	});
};
